#pragma once

#include <functional>
#include <ISprite_set.hxx>

namespace widgets
{

class Widget
{
public:
    Widget() = default;

    virtual ~Widget() = default;

protected:
    /// Returns the dimensions of the widget.
    virtual ge211::Dimensions dimensions() const = 0;

    /// Adds the widget to a sprite set.
    virtual void draw(ISprite_set&) const = 0;

    /// Override this to react to mouse events.
    virtual void mouse_down(ge211::Position)
    { }

    /// Override this to react to mouse events.
    virtual void mouse_up(ge211::Position)
    { }

    /// Override this to react to mouse events.
    virtual void mouse_enter(ge211::Position)
    { }

    /// Override this to react to mouse events.
    virtual void mouse_leave(ge211::Position)
    { }
};

class Interactive_widget : public Widget
{
public:
    /// Holds a client-provided function that can be called mouse events.
    struct Mouse_callback
    {
        Mouse_callback(std::function<void(ge211::Position)>);

        Mouse_callback(std::function<void()>);

        void operator()(ge211::Position position);

        std::function<void(ge211::Position)> fn;
    };

    /// Tells the widget to call `f` on mouse-down events.
    void on_mouse_down(Mouse_callback f);

    /// Tells the widget to call `f` on mouse-up events.
    void on_mouse_up(Mouse_callback f);

    /// Tells the widget to call `f` on mouse-enter events.
    void on_mouse_enter(Mouse_callback f);

    /// Tells the widget to call `f` on mouse-leave events.
    void on_mouse_leave(Mouse_callback f);

protected:
    void mouse_up(ge211::Position) override;

    void mouse_down(ge211::Position) override;

    void mouse_enter(ge211::Position) override;

    void mouse_leave(ge211::Position) override;

private:
    Mouse_callback on_mouse_down_, on_mouse_up_,
                   on_mouse_enter_, on_mouse_leave_;
};

class Button_widget : public Widget
{
public:
    Button_widget(const std::string& label, int size);

private:
    ge211::Font             sans_;
    ge211::Text_sprite      fg_;
    ge211::Rectangle_sprite bg_;
    ge211::Rectangle_sprite bg_hover_;
    ge211::Rectangle_sprite bg_active_;

    bool mouse_inside_ = false;
    bool mouse_down_   = false;
};

} // end namespace widgets
